@@ -19,7 +19,8 @@ import traceback
 import importlib
 from collections import OrderedDict
 
-from tpPyUtils import python, decorators, path as path_utils
+import tpDcc as tp
+from tpDcc.libs.python import python, decorators, strings, path as path_utils
 
 if python.is_python2():
     import pkgutil as loader
@@ -28,7 +29,7 @@ else:
 
 import artellapipe.register
 from artellapipe.utils import exceptions
-from artellapipe.core import config
+from artellapipe.core import defines
 from artellapipe.libs import artella as artella_lib
 from artellapipe.libs.artella.core import artellalib, artellaclasses
 
@@ -38,7 +39,6 @@ LOGGER = logging.getLogger()
 class ArtellaAssetsManager(object):
     def __init__(self):
         self._project = None
-
         self._assets = list()
         self._config = None
         self._registered_asset_classes = list()
@@ -59,6 +59,10 @@ class ArtellaAssetsManager(object):
     def assets(self):
         return self._assets
 
+    @property
+    def must_file_types(self):
+        return self._config.get('must_file_types', default=list())
+
     def set_project(self, project):
         """
         Sets the project this manager belongs to
@@ -66,7 +70,12 @@ class ArtellaAssetsManager(object):
         """
 
         self._project = project
-        self._config = config.get_config(project, 'artellapipe-assets')
+        self._config = tp.ConfigsMgr().get_config(
+            config_name='artellapipe-assets',
+            package_name=self._project.get_clean_name(),
+            root_package_name='artellapipe',
+            environment=project.get_environment()
+        )
 
         self._register_asset_classes()
 
@@ -84,6 +93,50 @@ class ArtellaAssetsManager(object):
         self._registered_asset_classes.append(asset_class)
         return True
 
+    def get_asset_categories(self):
+        """
+        Returns all asset categories of current project
+        :return: list(str)
+        """
+
+        return self.config.get('types', default=list())
+
+    def get_asset_id_from_node(self, node):
+        """
+        Returns asset name of the given node
+        :param node: str
+        :return: str
+        """
+
+        node_name = tp.Dcc.node_short_name(node)
+        namespace = tp.Dcc.node_namespace(node=node_name, check_node=False)
+        if not namespace:
+            # LOGGER.warning('Was not possible to find asset name from "{}"'.format(node))
+            return False
+
+        if namespace.startswith(':'):
+            namespace = namespace[1:]
+
+        return namespace
+
+    def get_asset_from_node(self, node):
+        """
+        Returns Asset from given node
+        :param node: str
+        :return: ArtellaAsset
+        """
+
+        asset_name = self.get_asset_id_from_node(node=node)
+        if not asset_name:
+            return None
+
+        asset = self.find_asset(asset_name=asset_name)
+        if not asset:
+            LOGGER.warning('No asset found for node: {} | {}'.format(node, asset_name))
+            return None
+
+        return asset
+
     def get_asset_types(self):
         """
         Returns a list with all available asset types
@@ -114,12 +167,55 @@ class ArtellaAssetsManager(object):
 
         return self.config.get('default_name', default='New Asset')
 
+    def get_shading_file_type(self):
+        """
+        Returns shading file type used by shaders
+        :return: str
+        """
+
+        return self.config.get('shading_file_type', default='shading')
+
+    def get_shaders_mapping_file_type(self):
+        """
+        Returns shader mapping file type
+        :return: str
+        """
+
+        return self.config.get('shaders_mapping_file_type', default='shadersmapping')
+
+    def get_default_asset_thumb(self):
+        """
+        Returns the default thumb used by assets
+        :return: str
+        """
+
+        return self.config.get('default_thumb', default='default')
+
     def get_assets_by_type(self, asset_type):
+        """
+        Returns asset by its type
+        :param asset_type: str
+        :return: ArtellaAsset
+        """
 
         if not self.is_valid_asset_type(asset_type):
             return None
 
-        return [asset for asset in self.assets if asset.ASSET_TYPE == asset_type]
+        return [asset for asset in self.assets if asset.FILE_TYPE == asset_type]
+
+    def open_asset_shaders_file(self, asset):
+        """
+        Open asset shaders file of the given asset
+        :param asset:
+        """
+
+        shading_file_type = artellapipe.AssetsMgr().get_shading_file_type()
+        file_path = asset.get_file(
+            file_type=shading_file_type, status=defines.ArtellaFileStatus.WORKING, fix_path=True)
+        valid_open = self._asset.open_file(file_type=shading_file_type, status=defines.ArtellaFileStatus.WORKING)
+        if not valid_open:
+            LOGGER.warning('Impossible to open Asset Shading File: {}'.format(file_path))
+            return None
 
     def get_assets_path(self):
         """
@@ -146,34 +242,37 @@ class ArtellaAssetsManager(object):
 
         return True
 
-    def find_all_assets(self, force=False):
+    def find_all_assets(self, force_update=False, force_login=True):
         """
         Returns a list of all assets in the project
-        :param force: bool, Whether assets cache updated must be forced or not
+        :param force_update: bool, Whether assets cache updated must be forced or not
+        :param force_login: bool, Whether logging to production tracker is forced or not
         :return: variant, ArtellaAsset or list(ArtellaAsset)
         """
 
         self._check_project()
 
-        assets_path = self.get_assets_path()
-        if not self.is_valid_assets_path():
-            LOGGER.warning('Impossible to retrieve assets from invalid path: {}'.format(assets_path))
-            return
-
-        if self._assets and not force:
+        if self._assets and not force_update:
             return self._assets
-        else:
-            python.clear_list(self._assets)
-            tracker = artellapipe.Tracker()
-            assets_dict = tracker.all_project_assets()
-            if not assets_dict:
-                LOGGER.warning("No assets found in current project!")
-                return None
-            for asset_data in assets_dict:
-                new_asset = self.create_asset(asset_data)
-                self._assets.append(new_asset)
 
-            return self._assets
+        python.clear_list(self._assets)
+
+        if not artellapipe.Tracker().is_logged() and force_login:
+            artellapipe.Tracker().login()
+        if not artellapipe.Tracker().is_logged():
+            LOGGER.warning(
+                'Impossible to find assets of current project because user is not log into production tracker')
+            return None
+        tracker = artellapipe.Tracker()
+        assets_list = tracker.all_project_assets()
+        if not assets_list:
+            LOGGER.warning("No assets found in current project!")
+            return None
+        for asset_data in assets_list:
+            new_asset = self.create_asset(asset_data)
+            self._assets.append(new_asset)
+
+        return self._assets
 
     def find_asset(self, asset_name=None, asset_path=None, allow_multiple_instances=False, force=False):
         """
@@ -189,11 +288,14 @@ class ArtellaAssetsManager(object):
         self._check_project()
 
         assets_found = list()
-        all_assets = self.find_all_assets(force=force)
+        all_assets = self.find_all_assets(force_update=force) or list()
         for asset in all_assets:
             if asset.get_name() == asset_name:
                 assets_found.append(asset)
 
+        if not assets_found:
+            return None
+
         if len(assets_found) > 1:
             LOGGER.warning('Found Multiple instances of Asset "{} | {}"'.format(asset_name, asset_path))
             if not allow_multiple_instances:
@@ -216,7 +318,7 @@ class ArtellaAssetsManager(object):
         else:
             asset_classes = self.asset_classes
             for asset_class in asset_classes:
-                if asset_class.ASSET_TYPE == category:
+                if asset_class.FILE_TYPE == category:
                     return asset_class(project=self._project, asset_data=asset_data)
 
     def create_asset_in_artella(self, asset_name, asset_path, folders_to_create=None):
@@ -250,15 +352,6 @@ class ArtellaAssetsManager(object):
 
         return asset_type in self.asset_types
 
-    def is_valid_asset_file_type(self, file_type):
-        """
-        Returns whether the current file type is valid or not for current project
-        :param file_type: str
-        :return: bool
-        """
-
-        return file_type in artellapipe.FilesMgr().files
-
     def get_asset_file(self, file_type, extension=None):
         """
         Returns asset file object class linked to given file type for current project
@@ -269,7 +362,7 @@ class ArtellaAssetsManager(object):
 
         self._check_project()
 
-        if not self.is_valid_asset_file_type(file_type):
+        if not artellapipe.FilesMgr().is_valid_file_type(file_type):
             return
 
         asset_file_class_found = None
@@ -284,6 +377,23 @@ class ArtellaAssetsManager(object):
 
         return asset_file_class_found
 
+    def get_asset_thumbnail_path(self, asset_name, create_folder=True):
+        """
+        Returns path where asset thumbnail is or should be located
+        If the folder does not exists, the folder will be created
+        :param create_folder: bool
+        :return: str
+        """
+
+        self._check_project()
+
+        data_path = self._project.get_data_path()
+        thumbnails_cache_folder = os.path.join(data_path, 'asset_thumbs_cache')
+        if not os.path.isdir(thumbnails_cache_folder) and create_folder:
+            os.makedirs(thumbnails_cache_folder)
+
+        return os.path.join(thumbnails_cache_folder, asset_name + '.png')
+
     def get_asset_data_file_path(self, asset_path):
         """
         Returns asset data file path of given asset
@@ -342,6 +452,155 @@ class ArtellaAssetsManager(object):
 
         return latest_version
 
+    @decorators.timestamp
+    def get_scene_assets(self, as_nodes=True, allowed_types=None, allowed_tags=None, node_id=None):
+        """
+        Returns a list with all assets in the current scene
+        :param as_nodes: bool, Whether to return found assets as ArtellaAssetNodes or as strings (asset names)
+        :param allowed_types: list(str), List of types asset to filter by. If None, no filter is done
+        :param allowed_tags: list(str), List of asset tags to filter by. If None, no filter is done
+        :return: list(str) or list(ArtellaAssetNode)
+        """
+
+        if not allowed_types:
+            allowed_types = list()
+
+        if not allowed_tags:
+            allowed_tags = list()
+
+        scene_assets = list()
+
+        all_assets = self.find_all_assets()
+        all_ids = dict()
+        for asset in all_assets:
+            all_ids[asset.get_id()] = asset
+
+        if not allowed_types and not allowed_tags:
+            valid_assets = all_assets
+        else:
+            valid_assets = list()
+            for asset in all_assets:
+                asset_category = asset.get_category()
+                asset_tags = asset.get_tags() or list()
+                if asset.FILE_TYPE in allowed_types or asset_category in allowed_types \
+                        or asset.FILE_TYPE in allowed_tags:
+                    valid_assets.append(asset)
+                    continue
+                for tag in asset_tags:
+                    if tag in allowed_tags:
+                        valid_assets.append(asset)
+                        break
+
+        if not valid_assets:
+            LOGGER.warning('No valid assets found in current scene!')
+            return
+
+        all_namespaces = tp.Dcc.list_namespaces()
+        for namespace in all_namespaces:
+            clean_namespace = namespace[1:] if namespace.startswith(':') else namespace
+            clean_namespace = strings.remove_digits_from_end_of_string(clean_namespace)
+            if clean_namespace not in all_ids:
+                continue
+            if node_id and namespace != node_id:
+                continue
+
+            namespace_nodes = tp.Dcc.all_nodes_in_namespace(namespace)
+            if not namespace_nodes:
+                LOGGER.warning('Namespace "{}" is empty!'.format(namespace_nodes))
+                continue
+            for namespace_node in namespace_nodes:
+                split_name = '|{}'.format(namespace)
+                if split_name not in namespace_node:
+                    continue
+                split = namespace_node.split('|{}:'.format(namespace))
+                root_node = strings.lstrips(namespace_node, '{}'.format(split[0])).split('|')[1]
+                asset_node = all_ids[clean_namespace]
+                if as_nodes:
+                    asset_node = artellapipe.AssetNode(
+                        project=self._project, node=root_node, asset=asset_node, id=namespace
+                    )
+                else:
+                    asset_node = root_node
+                if node_id and asset_node:
+                    return asset_node
+
+                scene_assets.append(asset_node)
+
+                break
+
+        return scene_assets
+
+    def get_asset_node_in_scene(self, node_id):
+        """
+        Returns given node wrapped in AssetNode (if exists)
+        :param node_id: str
+        :return: ArtellaAssetNode
+        """
+
+        self._check_project()
+
+        ns = self.get_asset_id_from_node(node_id)
+        if ns:
+            node_id = ns
+        if not node_id:
+            return None
+
+        asset_node = self.get_scene_assets(node_id=node_id)
+
+        return asset_node
+
+    def get_assets_in_shot(self, shot, force_login=True):
+        """
+        Returns all the assets contained in given shot breakdown defined in production tracker
+        :param shot:
+        :return:
+        """
+
+        if not artellapipe.Tracker().is_logged() and force_login:
+            artellapipe.Tracker().login()
+        if not artellapipe.Tracker().is_logged():
+            LOGGER.warning(
+                'Impossible to find assets of current project because user is not log into production tracker')
+            return None
+        tracker = artellapipe.Tracker()
+        assets_in_shots = tracker.all_assets_in_shot(shot)
+        if not assets_in_shots:
+            LOGGER.warning('No assets found in shot breakdown')
+            return None
+
+        found_assets = list()
+        for asset_data in assets_in_shots:
+            new_asset = self.create_asset(asset_data)
+            found_assets.append(new_asset)
+
+        return found_assets
+
+    def get_asset_renderable_shapes(self, asset, remove_namespace=False, full_path=True):
+        """
+        Returns a list with all renderable shapes of the given asset
+        :param remove_namespace: bool
+        :return: list(str)
+        """
+
+        renderable_shapes = list()
+
+        transform_relatives = tp.Dcc.list_relatives(
+            node=asset.get_name(), all_hierarchy=True, full_path=full_path, relative_type='transform',
+            shapes=False, intermediate_shapes=False)
+
+        for obj in transform_relatives:
+            if not tp.Dcc.object_exists(obj):
+                continue
+            shapes = tp.Dcc.list_shapes(node=obj, full_path=full_path, intermediate_shapes=False)
+            if not shapes:
+                continue
+            renderable_shapes.extend(shapes)
+
+        if remove_namespace:
+            renderable_shapes = [tp.Dcc.node_name_without_namespace(shape) for shape in renderable_shapes]
+
+        return list(set(renderable_shapes))
+
     def _check_project(self):
         """
         Internal function that checks whether or not assets manager has a project set. If not an exception is raised
@@ -373,7 +632,7 @@ class ArtellaAssetsManager(object):
 
     def _register_asset_classes(self):
         """
-        Internal function that project asset classes
+        Internal function that  registers project asset classes
         """
 
         if not self._project:
@@ -409,8 +668,8 @@ class ArtellaAssetsManager(object):
                 LOGGER.warning('No Asset Class "{}" found in Module: "{}"'.format(asset_class, asset_module))
                 continue
 
-            obj.ASSET_TYPE = asset_type
-            obj.ASSET_FILES = asset_files
+            obj.FILE_TYPE = asset_type
+            obj.FILES = asset_files
 
             self.register_asset_class(obj)
 
